@@ -49,17 +49,25 @@ public:
 
     ~DBusMenuExporter();
 
+    /**
+     * Asks the matching DBusMenuImporter to activate @p action. For menus it
+     * means popup them, for items it means triggering the associated action.
+     */
+    void activateAction(QAction *action);
+
 protected:
     /**
      * Must extract the icon name for action. This is the name which will
      * be used to present the icon over DBus.
-     * Default implementation returns a null QString.
+     * Default implementation returns action->icon().name() when built on Qt
+     * >= 4.7 and a null string otherwise.
      */
     virtual QString iconNameForAction(QAction *action);
 
 private Q_SLOTS:
     void doUpdateActions();
     void doEmitLayoutUpdated();
+    void slotActionDestroyed(QObject*);
 
 private:
     Q_DISABLE_COPY(DBusMenuExporter)
