@@ -98,7 +98,8 @@ public:
     QDBusAbstractInterface *m_interface;
     QMenu *m_menu;
     QMap<QDBusPendingCallWatcher *, Task> m_taskForWatcher;
-    QMap<int, QAction *> m_actionForId;
+    typedef QMap<int, QPointer<QAction> > ActionForId;
+    ActionForId m_actionForId;
     QSignalMapper m_mapper;
     QTimer *m_pendingLayoutUpdateTimer;
 
@@ -142,10 +143,10 @@ public:
      * instead of QMap::value()) to avoid warnings about these properties in
      * updateAction()
      */
-    QAction *createAction(int id, const QVariantMap &_map)
+    QAction *createAction(int id, const QVariantMap &_map, QWidget *parent)
     {
         QVariantMap map = _map;
-        QAction *action = new QAction(0);
+        QAction *action = new QAction(parent);
         action->setProperty(DBUSMENU_PROPERTY_ID, id);
 
         QString type = map.take("type").toString();
@@ -154,8 +155,7 @@ public:
         }
 
         if (map.take("children-display").toString() == "submenu") {
-            // FIXME: Leak?
-            QMenu *menu = createMenu(0);
+            QMenu *menu = createMenu(parent);
             action->setMenu(menu);
         }
 
@@ -179,7 +179,7 @@ public:
             buttonAction->setFont(font);
             buttonAction->setEnabled(true);
 
-            QWidgetAction *action = new QWidgetAction(0);
+            QWidgetAction *action = new QWidgetAction(parent);
             action->setObjectName("kmenu_title");
             QToolButton *titleButton = new QToolButton(0);
             EventSniffer *eventSniffer = new EventSniffer(titleButton);
@@ -188,8 +188,8 @@ public:
             titleButton->setDown(true); // prevent hover style changes in some styles
             titleButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
             action->setDefaultWidget(titleButton);
-            return action;
         }
+
         return action;
     }
 
@@ -312,6 +312,8 @@ DBusMenuImporter::DBusMenuImporter(const QString &service, const QString &path,
         this, SLOT(slotLayoutUpdated(uint, int)));
     QDBusConnection::sessionBus().connect(service, path, DBUSMENU_INTERFACE, "ItemPropertyUpdated", "isv",
         this, SLOT(slotItemPropertyUpdated(int, const QString &, const QDBusVariant &)));
+    QDBusConnection::sessionBus().connect(service, path, DBUSMENU_INTERFACE, "ItemActivationRequested", "iu",
+        this, SLOT(slotItemActivationRequested(int, uint)));
 
     d->refresh(0);
 }
@@ -407,6 +409,13 @@ void DBusMenuImporter::slotItemPropertyUpdated(int id, const QString &key, const
     d->updateActionProperty(action, key, value.variant());
 }
 
+void DBusMenuImporter::slotItemActivationRequested(int id, uint /*timestamp*/)
+{
+    QAction *action = d->m_actionForId.value(id);
+    DMRETURN_IF_FAIL(action);
+    actionActivationRequested(action);
+}
+
 void DBusMenuImporter::GetPropertiesCallback(int id, QDBusPendingCallWatcher *watcher)
 {
     #ifdef BENCHMARK
@@ -451,8 +460,14 @@ void DBusMenuImporter::GetChildrenCallback(int parentId, QDBusPendingCallWatcher
     menu->clear();
 
     Q_FOREACH(const DBusMenuItem &dbusMenuItem, list) {
-        QAction *action = d->createAction(dbusMenuItem.id, dbusMenuItem.properties);
-        d->m_actionForId.insert(dbusMenuItem.id, action);
+        QAction *action = d->createAction(dbusMenuItem.id, dbusMenuItem.properties, menu);
+        DBusMenuImporterPrivate::ActionForId::Iterator it = d->m_actionForId.find(dbusMenuItem.id);
+        if (it == d->m_actionForId.end()) {
+            d->m_actionForId.insert(dbusMenuItem.id, action);
+        } else {
+            delete *it;
+            *it = action;
+        }
         menu->addAction(action);
 
         connect(action, SIGNAL(triggered()),
