@@ -43,6 +43,8 @@ QTEST_MAIN(DBusMenuExporterTest)
 static const char *TEST_SERVICE = "org.kde.dbusmenu-qt-test";
 static const char *TEST_OBJECT_PATH = "/TestMenuBar";
 
+Q_DECLARE_METATYPE(QList<int>)
+
 void DBusMenuExporterTest::init()
 {
     QVERIFY(QDBusConnection::sessionBus().registerService(TEST_SERVICE));
@@ -251,7 +253,7 @@ void DBusMenuExporterTest::testDynamicSubMenu()
     QMenu inputMenu;
     DBusMenuExporter exporter(TEST_OBJECT_PATH, &inputMenu);
     QAction *action = inputMenu.addAction("menu");
-    QMenu *subMenu = new QMenu;
+    QMenu *subMenu = new QMenu(&inputMenu);
     action->setMenu(subMenu);
     MenuFiller filler(subMenu);
     filler.addAction(new QAction("a1", subMenu));
@@ -441,6 +443,25 @@ void DBusMenuExporterTest::testDeleteExporterBeforeMenu()
     inputMenu.removeAction(a1);
 }
 
+void DBusMenuExporterTest::testUpdateAndDeleteSubMenu()
+{
+    // Create a menu with a submenu
+    QMenu inputMenu;
+    QMenu *subMenu = inputMenu.addMenu("menu");
+    QAction *a1 = subMenu->addAction("a1");
+
+    // Export it
+    QVERIFY(QDBusConnection::sessionBus().registerService(TEST_SERVICE));
+    DBusMenuExporter exporter(TEST_OBJECT_PATH, &inputMenu);
+
+    // Update a1 (which is in subMenu) and delete subMenu right after that. If
+    // DBusMenuExporter is not careful it will crash in the qWait() because it
+    // tries to send itemUpdated() for a1.
+    a1->setText("Not a menu anymore");
+    delete subMenu;
+    QTest::qWait(500);
+}
+
 void DBusMenuExporterTest::testMenuShortcut()
 {
     // Create a menu containing an action with a shortcut
@@ -487,4 +508,81 @@ void DBusMenuExporterTest::testMenuShortcut()
     }
 }
 
+void DBusMenuExporterTest::testGetGroupProperties()
+{
+    // Create a menu containing two actions
+    QMenu inputMenu;
+    QVERIFY(QDBusConnection::sessionBus().registerService(TEST_SERVICE));
+    DBusMenuExporter *exporter = new DBusMenuExporter(TEST_OBJECT_PATH, &inputMenu);
+
+    QAction *a1 = inputMenu.addAction("a1");
+    QAction *a2 = inputMenu.addAction("a2");
+
+    // Check exporter is on DBus
+    QDBusInterface iface(TEST_SERVICE, TEST_OBJECT_PATH);
+    QVERIFY2(iface.isValid(), qPrintable(iface.lastError().message()));
+
+    // Get item ids
+    QDBusReply<DBusMenuItemList> reply = iface.call("GetChildren", 0, QStringList());
+    QVERIFY2(reply.isValid(), qPrintable(reply.error().message()));
+
+    DBusMenuItemList list = reply.value();
+    QCOMPARE(list.count(), inputMenu.actions().count());
+
+    int id1 = list.at(0).id;
+    int id2 = list.at(1).id;
+
+    // Get group properties
+    QList<int> ids = QList<int>() << id1 << id2;
+    reply = iface.call("GetGroupProperties", QVariant::fromValue(ids), QStringList());
+    QVERIFY2(reply.isValid(), qPrintable(reply.error().message()));
+    list = reply.value();
+
+    // Check the info we received
+    QCOMPARE(list.count(), inputMenu.actions().count());
+
+    Q_FOREACH(const QAction* action, inputMenu.actions()) {
+        DBusMenuItem item = list.takeFirst();
+        QCOMPARE(item.properties.value("label").toString(), action->text());
+    }
+}
+
+void DBusMenuExporterTest::testActivateAction()
+{
+    // Create a menu containing two actions
+    QMenu inputMenu;
+    QVERIFY(QDBusConnection::sessionBus().registerService(TEST_SERVICE));
+    DBusMenuExporter *exporter = new DBusMenuExporter(TEST_OBJECT_PATH, &inputMenu);
+
+    QAction *a1 = inputMenu.addAction("a1");
+    QAction *a2 = inputMenu.addAction("a2");
+
+    // Check exporter is on DBus
+    QDBusInterface iface(TEST_SERVICE, TEST_OBJECT_PATH);
+    QVERIFY2(iface.isValid(), qPrintable(iface.lastError().message()));
+
+    ManualSignalSpy spy;
+    QDBusConnection::sessionBus().connect(TEST_SERVICE, TEST_OBJECT_PATH, "org.ayatana.dbusmenu", "ItemActivationRequested", "iu", &spy, SLOT(receiveCall(int, uint)));
+
+    // Get item ids
+    QDBusReply<DBusMenuItemList> reply = iface.call("GetChildren", 0, QStringList());
+    QVERIFY2(reply.isValid(), qPrintable(reply.error().message()));
+
+    DBusMenuItemList list = reply.value();
+    QCOMPARE(list.count(), inputMenu.actions().count());
+
+    int id1 = list.at(0).id;
+    int id2 = list.at(1).id;
+
+    // Trigger actions
+    exporter->activateAction(a1);
+    exporter->activateAction(a2);
+
+    // Check we received the signals in the correct order
+    QTest::qWait(500);
+    QCOMPARE(spy.count(), 2);
+    QCOMPARE(spy.takeFirst().at(0).toInt(), id1);
+    QCOMPARE(spy.takeFirst().at(0).toInt(), id2);
+}
+
 #include "dbusmenuexportertest.moc"
