@@ -44,7 +44,10 @@ private Q_SLOTS:
     void testNonExclusiveActionGroup();
     void testClickDeletedAction();
     void testDeleteExporterBeforeMenu();
+    void testUpdateAndDeleteSubMenu();
     void testMenuShortcut();
+    void testGetGroupProperties();
+    void testActivateAction();
 
     void init();
     void cleanup();
