@@ -61,6 +61,18 @@ public:
     }
 };
 
+Q_DECLARE_METATYPE(QAction*)
+
+void DBusMenuImporterTest::initTestCase()
+{
+    qRegisterMetaType<QAction*>("QAction*");
+}
+
+void DBusMenuImporterTest::cleanup()
+{
+    waitForDeferredDeletes();
+}
+
 void DBusMenuImporterTest::testStandardItem()
 {
     RegisterServiceHelper helper;
@@ -200,4 +212,76 @@ void DBusMenuImporterTest::testDynamicMenu()
     QCOMPARE(spyOld.count(), 1);
 }
 
+void DBusMenuImporterTest::testActionActivationRequested()
+{
+    RegisterServiceHelper helper;
+
+    // Export a menu
+    QMenu inputMenu;
+    QAction *inputA1 = inputMenu.addAction("a1");
+    QAction *inputA2 = inputMenu.addAction("a2");
+    DBusMenuExporter exporter(TEST_OBJECT_PATH, &inputMenu);
+
+    // Import the menu
+    DBusMenuImporter importer(TEST_SERVICE, TEST_OBJECT_PATH);
+    QSignalSpy spy(&importer, SIGNAL(actionActivationRequested(QAction*)));
+
+    QTest::qWait(500);
+    QMenu *outputMenu = importer.menu();
+
+    // Get matching output actions
+    QCOMPARE(outputMenu->actions().count(), 2);
+    QAction *outputA1 = outputMenu->actions().at(0);
+    QAction *outputA2 = outputMenu->actions().at(1);
+
+    // Request activation
+    exporter.activateAction(inputA1);
+    exporter.activateAction(inputA2);
+
+    // Check we received the signal in the right order
+    QTest::qWait(500);
+    QCOMPARE(spy.count(), 2);
+    QCOMPARE(spy.takeFirst().at(0).value<QAction*>(), outputA1);
+    QCOMPARE(spy.takeFirst().at(0).value<QAction*>(), outputA2);
+}
+
+void DBusMenuImporterTest::testActionsAreDeletedWhenImporterIs()
+{
+    RegisterServiceHelper helper;
+
+    // Export a menu
+    QMenu inputMenu;
+    inputMenu.addAction("a1");
+    QMenu *inputSubMenu = inputMenu.addMenu("subMenu");
+    inputSubMenu->addAction("a2");
+    DBusMenuExporter exporter(TEST_OBJECT_PATH, &inputMenu);
+
+    // Import the menu
+    DBusMenuImporter *importer = new DBusMenuImporter(TEST_SERVICE, TEST_OBJECT_PATH);
+    QTest::qWait(500);
+
+    // Put all items of the menu in a list of QPointers
+    QList< QPointer<QObject> > children;
+
+    QMenu *outputMenu = importer->menu();
+    QCOMPARE(outputMenu->actions().count(), 2);
+    QMenu *outputSubMenu = outputMenu->actions().at(1)->menu();
+    QVERIFY(outputSubMenu);
+    QCOMPARE(outputSubMenu->actions().count(), 1);
+
+    children << outputMenu->actions().at(0);
+    children << outputMenu->actions().at(1);
+    children << outputSubMenu;
+    children << outputSubMenu->actions().at(0);
+
+    delete importer;
+    waitForDeferredDeletes();
+
+    // There should be only invalid pointers in children
+    Q_FOREACH(QPointer<QObject> child, children) {
+        //qDebug() << child;
+        QVERIFY(child.isNull());
+    }
+}
+
 #include "dbusmenuimportertest.moc"
