@@ -33,11 +33,16 @@ class DBusMenuImporterTest : public QObject
 {
 Q_OBJECT
 private Q_SLOTS:
+    void cleanup();
     void testStandardItem();
     void testAddingNewItem();
     void testShortcut();
     void testDeletingImporterWhileWaitingForAboutToShow();
     void testDynamicMenu();
+    void testActionActivationRequested();
+    void testActionsAreDeletedWhenImporterIs();
+
+    void initTestCase();
 };
 
 #endif /* DBUSMENUIMPORTERTEST_H */
